'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var codemirror6Prerelease = require('@datavis-tech/codemirror-6-prerelease');

const changeToOps = (path, transaction) => change => {
  const ops = [];
  const p = path.concat([change.from]);

  // String delete
  if (change.from !== change.to) {
    ops.push({
      p,
      sd: transaction.startState.doc.slice(change.from, change.to)
    });
  }

  // String insert
  const joined = change.text.join('\n');
  if (joined.length) {
    ops.push({
      p,
      si: joined
    });
  }
  return ops;
};

const transactionToOps = (path, transaction) =>
  transaction.changes.changes
    .map(changeToOps(path, transaction))
    .reduce((accumulator, ops) => accumulator.concat(ops), []);

// This file exists so that it's easy to change the package

const siToText = si => si.split('\n');

const opToChange = (transaction, op) => {
  const from = op.p[op.p.length - 1];

  // String insert
  if (op.si !== undefined) {
    return transaction.change(new codemirror6Prerelease.Change(from, from, siToText(op.si)));
  }

  // String delete
  if (op.sd !== undefined) {
    return transaction.change(new codemirror6Prerelease.Change(from, from + op.sd.length, ['']));
  }

  throw new Error('Invalid string op.');
};

const opsToTransaction = (path, state, ops) =>
  ops.reduce(opToChange, state.t());

const ot = (path, emitOps) =>
  new codemirror6Prerelease.StateField({
    init: () => ({}),
    apply: (transaction, state) => {
      const ops = transactionToOps(path, transaction);
      if (ops.length > 0) {
        emitOps(ops);
      }
      return state;
    },
    debugName: 'ot'
  }).extension;

exports.opsToTransaction = opsToTransaction;
exports.ot = ot;
exports.transactionToOps = transactionToOps;
//# sourceMappingURL=codemirror-ot.js.map
